@@ -6,8 +6,9 @@
 ### - abs, sign, sqrt, floor, ceiling, trunc
 ### - cummax, cummin, cumprod, cumsum
 ### - log, log10, log2, log1p, exp, expm1
-### - cos, cospi, acos, sin, sinpi, asin, tan, tanpi, atan
-### - cosh, acosh, sinh, asinh, tanh, atanh
+### - sin, asin, sinh, asinh, sinpi,
+### - cos, acos, cosh, acosh, cospi,
+### - tan, atan, tanh, atanh, tanpi,
 ### - gamma, lgamma, digamma, trigamma
 ###
 ### The 'Math2' group consists of the following methods: round, signif
@@ -19,17 +20,18 @@
 ### 'Math' group
 ###
 
-### We only support functions in the 'Math' group that preserve sparsity.
-.SUPPORTED_MATH_OPS <- c(
+### SparseArray objects only support functions from the 'Math' group that
+### propagate zeros.
+SUPPORTED_MATH_OPS <- c(
     "abs", "sign", "sqrt", "floor", "ceiling", "trunc",
     "log1p", "expm1",
-    "sin", "sinpi", "asin", "tan", "tanpi", "atan",
-    "sinh", "asinh", "tanh", "atanh"
+    "sin", "asin", "sinh", "asinh", "sinpi",
+    "tan", "atan", "tanh", "atanh", "tanpi"
 )
 
 .check_Math_op <- function(op)
 {
-    if (!(op %in% .SUPPORTED_MATH_OPS))
+    if (!(op %in% SUPPORTED_MATH_OPS))
         stop(wmsg(op, "() is not supported on SparseArray ",
                   "objects (result wouldn't be sparse in general)"))
 }
@@ -42,8 +44,9 @@
     if (type(x) != "double")
         stop(wmsg("the ", op, "() method for SVT_SparseArray objects ",
                   "only supports input of type \"double\" at the moment"))
-    ans_SVT <- SparseArray.Call("C_Math_SVT", x@dim, x@type, x@SVT, op, 0.0)
-    new_SVT_SparseArray(x@dim, x@dimnames, "double", ans_SVT, check=FALSE)
+    new_SVT <- SparseArray.Call("C_Math_SVT",
+                                x@dim, x@type, x@SVT, FALSE, op, 0.0)
+    BiocGenerics:::replaceSlots(x, type="double", SVT=new_SVT, check=FALSE)
 }
 
 setMethod("Math", "SVT_SparseArray", function(x) .Math_SVT(.Generic, x))
@@ -59,13 +62,14 @@ setMethod("Math", "SVT_SparseArray", function(x) .Math_SVT(.Generic, x))
     check_svt_version(x)
     if (type(x) != "double")
         stop(wmsg("the ", op, "() method for SVT_SparseArray objects ",
-                  "only supports input of type \"double\""))
+                  "only supports input of type \"double\" at the moment"))
     if (!isSingleNumber(digits))
         stop(wmsg("'digits' must be a single number"))
     if (!is.double(digits))
         digits <- as.double(digits)
-    ans_SVT <- SparseArray.Call("C_Math_SVT", x@dim, x@type, x@SVT, op, digits)
-    new_SVT_SparseArray(x@dim, x@dimnames, "double", ans_SVT, check=FALSE)
+    new_SVT <- SparseArray.Call("C_Math_SVT",
+                                x@dim, x@type, x@SVT, FALSE, op, digits)
+    BiocGenerics:::replaceSlots(x, type="double", SVT=new_SVT, check=FALSE)
 }
 
 setMethod("round", "SVT_SparseArray",
