@@ -19,12 +19,14 @@
                   #"\"integer\", \"complex\", or \"logical\""))
 }
 
-.crossprod2_SVT_mat <- function(x, y, transpose.y=FALSE)
+.crossprod2_SparseMatrix_matrix <- function(x, y, transpose.y=FALSE)
 {
-    stopifnot(is(x, "SVT_SparseMatrix"),
-              is.matrix(y),
-              isTRUEorFALSE(transpose.y))
-    check_svt_version(x)
+    if (is(x, "SVT_SparseMatrix")) {
+        check_svt_version(x)
+    } else {
+        x <- as(x, "SVT_SparseMatrix")
+    }
+    stopifnot(is.matrix(y), isTRUEorFALSE(transpose.y))
     if (transpose.y) {
         if (nrow(x) != ncol(y))
             stop(wmsg("non-conformable arguments"))
@@ -50,12 +52,14 @@
                      ans_type, ans_dimnames)
 }
 
-.crossprod2_mat_SVT <- function(x, y, transpose.x=FALSE)
+.crossprod2_matrix_SparseMatrix <- function(x, y, transpose.x=FALSE)
 {
-    stopifnot(is.matrix(x),
-              is(y, "SVT_SparseMatrix"),
-              isTRUEorFALSE(transpose.x))
-    check_svt_version(y)
+    if (is(y, "SVT_SparseMatrix")) {
+        check_svt_version(y)
+    } else {
+        y <- as(y, "SVT_SparseMatrix")
+    }
+    stopifnot(is.matrix(x), isTRUEorFALSE(transpose.x))
     if (transpose.x) {
         if (ncol(x) != nrow(y))
             stop(wmsg("non-conformable arguments"))
@@ -81,11 +85,18 @@
                      ans_type, ans_dimnames)
 }
 
-.crossprod2_SVT_SVT <- function(x, y=NULL)
+.crossprod2_SparseMatrix_SparseMatrix <- function(x, y=NULL)
 {
-    stopifnot(is(x, "SVT_SparseMatrix"), is(y, "SVT_SparseMatrix"))
-    check_svt_version(x)
-    check_svt_version(y)
+    if (is(x, "SVT_SparseMatrix")) {
+        check_svt_version(x)
+    } else {
+        x <- as(x, "SVT_SparseMatrix")
+    }
+    if (is(y, "SVT_SparseMatrix")) {
+        check_svt_version(y)
+    } else {
+        y <- as(y, "SVT_SparseMatrix")
+    }
     if (nrow(x) != nrow(y))
         stop(wmsg("non-conformable arguments"))
     ans_dim <- c(ncol(x), ncol(y))
@@ -104,10 +115,14 @@
                      ans_type, ans_dimnames)
 }
 
-.crossprod1_SVT <- function(x, y=NULL)
+.crossprod1_SparseMatrix <- function(x, y=NULL)
 {
-    stopifnot(is(x, "SVT_SparseMatrix"), is.null(y))
-    check_svt_version(x)
+    if (is(x, "SVT_SparseMatrix")) {
+        check_svt_version(x)
+    } else {
+        x <- as(x, "SVT_SparseMatrix")
+    }
+    stopifnot(is.null(y))
     ans_dim <- c(ncol(x), ncol(x))
     .check_crossprod_input_type(type(x))
     ans_type <- "double"
@@ -118,28 +133,28 @@
                      ans_type, ans_dimnames)
 }
 
-setMethod("crossprod", c("SVT_SparseMatrix", "matrix"),
-    .crossprod2_SVT_mat
+setMethod("crossprod", c("SparseMatrix", "matrix"),
+    .crossprod2_SparseMatrix_matrix
 )
 
-setMethod("crossprod", c("matrix", "SVT_SparseMatrix"),
-    .crossprod2_mat_SVT
+setMethod("crossprod", c("matrix", "SparseMatrix"),
+    .crossprod2_matrix_SparseMatrix
 )
 
-setMethod("crossprod", c("SVT_SparseMatrix", "SVT_SparseMatrix"),
-    .crossprod2_SVT_SVT
+setMethod("crossprod", c("SparseMatrix", "SparseMatrix"),
+    .crossprod2_SparseMatrix_SparseMatrix
 )
 
-setMethod("crossprod", c("SVT_SparseMatrix", "ANY"),
-    function(x, y=NULL) .crossprod2_SVT_SVT(x, as(y, "SVT_SparseMatrix"))
+setMethod("crossprod", c("SparseMatrix", "ANY"),
+    function(x, y=NULL) .crossprod2_SparseMatrix_SparseMatrix(x, y)
 )
 
-setMethod("crossprod", c("ANY", "SVT_SparseMatrix"),
-    function(x, y=NULL) .crossprod2_SVT_SVT(as(x, "SVT_SparseMatrix"), y)
+setMethod("crossprod", c("ANY", "SparseMatrix"),
+    function(x, y=NULL) .crossprod2_SparseMatrix_SparseMatrix(x, y)
 )
 
-setMethod("crossprod", c("SVT_SparseMatrix", "missing"),
-    .crossprod1_SVT
+setMethod("crossprod", c("SparseMatrix", "missing"),
+    .crossprod1_SparseMatrix
 )
 
 
@@ -147,28 +162,30 @@ setMethod("crossprod", c("SVT_SparseMatrix", "missing"),
 ### tcrossprod()
 ###
 
-setMethod("tcrossprod", c("SVT_SparseMatrix", "matrix"),
-    function(x, y=NULL) .crossprod2_SVT_mat(t(x), y, transpose.y=TRUE)
+setMethod("tcrossprod", c("SparseMatrix", "matrix"),
+    function(x, y=NULL) .crossprod2_SparseMatrix_matrix(t(x), y,
+                                                        transpose.y=TRUE)
 )
 
-setMethod("tcrossprod", c("matrix", "SVT_SparseMatrix"),
-    function(x, y=NULL) .crossprod2_mat_SVT(x, t(y), transpose.x=TRUE)
+setMethod("tcrossprod", c("matrix", "SparseMatrix"),
+    function(x, y=NULL) .crossprod2_matrix_SparseMatrix(x, t(y),
+                                                        transpose.x=TRUE)
 )
 
-setMethod("tcrossprod", c("SVT_SparseMatrix", "SVT_SparseMatrix"),
-    function(x, y=NULL) .crossprod2_SVT_SVT(t(x), t(y))
+setMethod("tcrossprod", c("SparseMatrix", "SparseMatrix"),
+    function(x, y=NULL) .crossprod2_SparseMatrix_SparseMatrix(t(x), t(y))
 )
 
-setMethod("tcrossprod", c("SVT_SparseMatrix", "ANY"),
-    function(x, y=NULL) .crossprod2_SVT_SVT(t(x), t(as(y, "SVT_SparseMatrix")))
+setMethod("tcrossprod", c("SparseMatrix", "ANY"),
+    function(x, y=NULL) .crossprod2_SparseMatrix_SparseMatrix(t(x), t(y))
 )
 
-setMethod("tcrossprod", c("ANY", "SVT_SparseMatrix"),
-    function(x, y=NULL) .crossprod2_SVT_SVT(t(as(x, "SVT_SparseMatrix")), t(y))
+setMethod("tcrossprod", c("ANY", "SparseMatrix"),
+    function(x, y=NULL) .crossprod2_SparseMatrix_SparseMatrix(t(x), t(y))
 )
 
-setMethod("tcrossprod", c("SVT_SparseMatrix", "missing"),
-    function(x, y=NULL) .crossprod1_SVT(t(x))
+setMethod("tcrossprod", c("SparseMatrix", "missing"),
+    function(x, y=NULL) .crossprod1_SparseMatrix(t(x))
 )
 
 
@@ -176,23 +193,23 @@ setMethod("tcrossprod", c("SVT_SparseMatrix", "missing"),
 ### Matrix multiplication
 ###
 
-setMethod("%*%", c("SVT_SparseMatrix", "matrix"),
-    function(x, y) .crossprod2_SVT_mat(t(x), y)
+setMethod("%*%", c("SparseMatrix", "matrix"),
+    function(x, y) .crossprod2_SparseMatrix_matrix(t(x), y)
 )
 
-setMethod("%*%", c("matrix", "SVT_SparseMatrix"),
-    function(x, y) .crossprod2_mat_SVT(x, y, transpose.x=TRUE)
+setMethod("%*%", c("matrix", "SparseMatrix"),
+    function(x, y) .crossprod2_matrix_SparseMatrix(x, y, transpose.x=TRUE)
 )
 
-setMethod("%*%", c("SVT_SparseMatrix", "SVT_SparseMatrix"),
-    function(x, y) .crossprod2_SVT_SVT(t(x), y)
+setMethod("%*%", c("SparseMatrix", "SparseMatrix"),
+    function(x, y) .crossprod2_SparseMatrix_SparseMatrix(t(x), y)
 )
 
-setMethod("%*%", c("SVT_SparseMatrix", "ANY"),
-    function(x, y) .crossprod2_SVT_SVT(t(x), as(y, "SVT_SparseMatrix"))
+setMethod("%*%", c("SparseMatrix", "ANY"),
+    function(x, y) .crossprod2_SparseMatrix_SparseMatrix(t(x), y)
 )
 
-setMethod("%*%", c("ANY", "SVT_SparseMatrix"),
-    function(x, y) .crossprod2_SVT_SVT(t(as(x, "SVT_SparseMatrix")), y)
+setMethod("%*%", c("ANY", "SparseMatrix"),
+    function(x, y) .crossprod2_SparseMatrix_SparseMatrix(t(x), y)
 )
 
