@@ -3,7 +3,7 @@
 
 #include <Rdefines.h>
 
-typedef struct extendable_jagged_array {
+typedef struct extendable_jagged_array_t {
 	size_t _ncol;
 	int **_cols;
 	size_t *_buflengths;
@@ -21,8 +21,8 @@ void _add_ExtendableJaggedArray_elt(
 );
 
 SEXP _move_ExtendableJaggedArrays_to_SVT(
-	ExtendableJaggedArray *offss,
-	ExtendableJaggedArray *valss
+	ExtendableJaggedArray *valss,
+	ExtendableJaggedArray *offss
 );
 
 #endif  /* _EXTENDABLE_JAGGED_ARRAY_H_ */
