@@ -18,16 +18,18 @@
 
 int _get_summarize_opcode(SEXP op, SEXPTYPE Rtype)
 {
-	if (!IS_CHARACTER(op) || LENGTH(op) != 1)
+	if (!(IS_CHARACTER(op) && LENGTH(op) == 1))
 		error("'op' must be a single string");
 	op = STRING_ELT(op, 0);
 	if (op == NA_STRING)
 		error("'op' cannot be NA");
 	const char *s = CHAR(op);
+
 	if (Rtype != LGLSXP && Rtype != INTSXP && Rtype != REALSXP &&
 	    Rtype != CPLXSXP && Rtype != STRSXP)
 		error("%s() does not support SparseArray objects "
 		      "of type() \"%s\"", s, type2char(Rtype));
+
 	if (strcmp(s, "anyNA") == 0)
 		return ANYNA_OPCODE;
 	if (strcmp(s, "countNAs") == 0)
@@ -47,8 +49,8 @@ int _get_summarize_opcode(SEXP op, SEXPTYPE Rtype)
 		return PROD_OPCODE;
 	if (strcmp(s, "mean") == 0)
 		return MEAN_OPCODE;
-	if (strcmp(s, "sum_centered_X2") == 0)
-		return SUM_CENTERED_X2_OPCODE;
+	if (strcmp(s, "centered_X2_sum") == 0)
+		return CENTERED_X2_SUM_OPCODE;
 	if (strcmp(s, "sum_X_X2") == 0)
 		return SUM_X_X2_OPCODE;
 	if (strcmp(s, "var1") == 0)
@@ -70,7 +72,7 @@ int _get_summarize_opcode(SEXP op, SEXPTYPE Rtype)
 	      "\"anyNA\", \"countNAs\", \"any\", \"all\",\n"
 	      "                       \"min\", \"max\", "
 	      "\"range\", \"sum\", \"prod\", \"mean\",\n"
-	      "                       \"sum_centered_X2\", \"sum_X_X2\",\n"
+	      "                       \"centered_X2_sum\", \"sum_X_X2\",\n"
 	      "                       \"var1\", \"var2\", \"sd1\", \"sd2\"");
 	return 0;
 }
@@ -122,7 +124,7 @@ void _init_SummarizeResult(const SummarizeOp *summarize_op,
 		res->outbuf.one_double[0] = 1.0;
 		res->postprocess_one_zero = 1;
 		return;
-	    case SUM_CENTERED_X2_OPCODE: case VAR1_OPCODE: case SD1_OPCODE:
+	    case CENTERED_X2_SUM_OPCODE: case VAR1_OPCODE: case SD1_OPCODE:
 		res->out_Rtype = REALSXP;
 		res->outbuf.one_double[0] = 0.0;
 		return;
@@ -196,11 +198,10 @@ static inline int anyNA_doubles(const double *x, int n, int outbuf[1])
 	return OUTBUF_IS_SET;
 }
 
-#define	RCOMPLEX_IS_NA(x) (ISNAN((x)->r) || ISNAN((x)->i))
 static inline int anyNA_Rcomplexes(const Rcomplex *x, int n, int outbuf[1])
 {
 	for (int i = 0; i < n; i++, x++) {
-		if (RCOMPLEX_IS_NA(x)) {
+		if (RCOMPLEX_IS_NA_OR_NaN(x)) {
 			/* Bail out early. */
 			outbuf[0] = 1;
 			return OUTBUF_IS_SET_WITH_BREAKING_VALUE;
@@ -249,7 +250,7 @@ static inline int countNAs_Rcomplexes(const Rcomplex *x, int n,
 {
 	double out0 = outbuf[0];
 	for (int i = 0; i < n; i++, x++) {
-		if (RCOMPLEX_IS_NA(x))
+		if (RCOMPLEX_IS_NA_OR_NaN(x))
 			out0++;
 	}
 	outbuf[0] = out0;
@@ -618,7 +619,7 @@ static inline int prod_doubles(const double *x, int n,
 }
 
 /* 'outbuf' initialized by _init_SummarizeResult() above. */
-static inline int sum_centered_X2_ints(const int *x, int n,
+static inline int centered_X2_sum_ints(const int *x, int n,
 		int na_rm, double center, R_xlen_t *nacount,
 		double outbuf[1])
 {
@@ -641,7 +642,7 @@ static inline int sum_centered_X2_ints(const int *x, int n,
 }
 
 /* 'outbuf' initialized by _init_SummarizeResult() above. */
-static inline int sum_centered_X2_doubles(const double *x, int n,
+static inline int centered_X2_sum_doubles(const double *x, int n,
 		int na_rm, double center, R_xlen_t *nacount,
 		double outbuf[1])
 {
@@ -808,7 +809,7 @@ static int summarize_ones(SEXPTYPE x_Rtype, int x_len,
 		res->outbuf.one_double[0] += (double) x_len;
 	    case PROD_OPCODE:
 		return OUTBUF_IS_SET;
-	    case SUM_CENTERED_X2_OPCODE: case VAR1_OPCODE: case SD1_OPCODE: {
+	    case CENTERED_X2_SUM_OPCODE: case VAR1_OPCODE: case SD1_OPCODE: {
 		double delta = 1.0 - center;
 		res->outbuf.one_double[0] += delta * delta * x_len;
 		return OUTBUF_IS_SET;
@@ -853,8 +854,8 @@ static int summarize_ints(const int *x, int x_len,
 	    case PROD_OPCODE:
 		return prod_ints(x, x_len, na_rm, nacount_p,
 				res->outbuf.one_double);
-	    case SUM_CENTERED_X2_OPCODE: case VAR1_OPCODE: case SD1_OPCODE:
-		return sum_centered_X2_ints(x, x_len, na_rm,
+	    case CENTERED_X2_SUM_OPCODE: case VAR1_OPCODE: case SD1_OPCODE:
+		return centered_X2_sum_ints(x, x_len, na_rm,
 				center, nacount_p,
 				res->outbuf.one_double);
 	    case SUM_X_X2_OPCODE: case VAR2_OPCODE: case SD2_OPCODE:
@@ -890,8 +891,8 @@ static int summarize_doubles(const double *x, int x_len,
 	    case PROD_OPCODE:
 		return prod_doubles(x, x_len, na_rm, nacount_p,
 				res->outbuf.one_double);
-	    case SUM_CENTERED_X2_OPCODE: case VAR1_OPCODE: case SD1_OPCODE:
-		return sum_centered_X2_doubles(x, x_len, na_rm,
+	    case CENTERED_X2_SUM_OPCODE: case VAR1_OPCODE: case SD1_OPCODE:
+		return centered_X2_sum_doubles(x, x_len, na_rm,
 				center, nacount_p,
 				res->outbuf.one_double);
 	    case SUM_X_X2_OPCODE: case VAR2_OPCODE: case SD2_OPCODE:
@@ -1028,24 +1029,82 @@ static void summarize_one_zero(const SummarizeOp *summarize_op,
 	return;
 }
 
-
-void _postprocess_SummarizeResult(const SummarizeOp *summarize_op,
-				  SummarizeResult *res)
+/* Does NOT increase 'res->in_length' by 1. */
+static void summarize_one_NA(const SummarizeOp *summarize_op,
+			     SummarizeResult *res)
 {
-	int opcode = summarize_op->opcode;
-	R_xlen_t zerocount = res->in_length - res->in_nzcount;
-	R_xlen_t effective_len = res->in_length;
-
+	if (res->outbuf_status == OUTBUF_IS_SET_WITH_BREAKING_VALUE)
+		error("SparseArray internal error in summarize_one_NA():\n"
+		      "    outbuf already set with breaking value");
 	if (summarize_op->na_rm)
-		effective_len -= res->in_nacount;
+		return;
+	int new_status;
+	switch (summarize_op->in_Rtype) {
+	    case INTSXP: case LGLSXP: {
+		new_status = summarize_ints(&intNA, 1,
+				summarize_op->opcode, 0,
+				summarize_op->center, res);
+		break;
+	    }
+	    case REALSXP: {
+		new_status = summarize_doubles(&doubleNA, 1,
+				summarize_op->opcode, 0,
+				summarize_op->center, res);
+		break;
+	    }
+	    case CPLXSXP: {
+		new_status = summarize_Rcomplexes(&RcomplexNA, 1,
+				summarize_op->opcode, 0,
+				summarize_op->center, res);
+		break;
+	    }
+	    case STRSXP: {
+		SEXP val = PROTECT(ScalarString(NA_STRING));
+		new_status = summarize_Rstrings(val,
+				summarize_op->opcode, 0,
+				summarize_op->center, res);
+		UNPROTECT(1);
+		break;
+	    }
+	    default:
+		error("SparseArray internal error in summarize_one_NA():\n"
+		      "    input type \"%s\" is not supported",
+		      type2char(summarize_op->in_Rtype));
+	}
+	res->outbuf_status = new_status;
+	return;
+}
 
-	if (res->postprocess_one_zero && zerocount != 0)
-		summarize_one_zero(summarize_op, res);
-
-	/* Nothing else to do if a break condition was reached. */
+void _postprocess_SummarizeResult(SummarizeResult *res, int na_background,
+				  const SummarizeOp *summarize_op)
+{
+	/* There's nothing to do if a break condition was reached. */
 	if (res->outbuf_status == OUTBUF_IS_SET_WITH_BREAKING_VALUE)
 		return;
 
+	int opcode = summarize_op->opcode;
+	R_xlen_t zerocount = res->in_length - res->in_nzcount;
+	if (opcode == COUNTNAS_OPCODE) {
+		if (na_background)
+			res->outbuf.one_double[0] += zerocount;
+		return;
+	}
+	R_xlen_t effective_len = res->in_length;
+
+	if (summarize_op->na_rm) {
+		if (na_background)
+			effective_len = res->in_nzcount;
+		effective_len -= res->in_nacount;
+	}
+
+	if (zerocount != 0) {
+		if (na_background) {
+			summarize_one_NA(summarize_op, res);
+		} else if (res->postprocess_one_zero) {
+			summarize_one_zero(summarize_op, res);
+		}
+	}
+
 	if (res->outbuf_status == OUTBUF_IS_NOT_SET) {
 		if ((opcode == MIN_OPCODE || opcode == MAX_OPCODE ||
 		     opcode == RANGE_OPCODE) &&
@@ -1055,7 +1114,7 @@ void _postprocess_SummarizeResult(const SummarizeOp *summarize_op,
 			   has length 0 (i.e. 'res->in_length == 0'), or if
 			   it contains only NAs (i.e. 'res->in_nacount ==
 			   res->in_length') and 'summarize_op->na_rm' is True.
-			   This is a case where we intentional deviate from
+			   This is a case where we intentionally deviate from
 			   base::min(), base::max(), and base::range(). */
 			if (opcode == RANGE_OPCODE) {
 				res->outbuf.two_ints[0] =
@@ -1081,10 +1140,12 @@ void _postprocess_SummarizeResult(const SummarizeOp *summarize_op,
 		res->outbuf.one_double[0] /= (double) effective_len;
 		return;
 	    }
-	    case SUM_CENTERED_X2_OPCODE: case VAR1_OPCODE: case SD1_OPCODE: {
+	    case CENTERED_X2_SUM_OPCODE: case VAR1_OPCODE: case SD1_OPCODE: {
 		double center = summarize_op->center;
-		res->outbuf.one_double[0] += center * center * zerocount;
-		if (opcode == SUM_CENTERED_X2_OPCODE)
+		if (!na_background)
+			res->outbuf.one_double[0] +=
+				center * center * zerocount;
+		if (opcode == CENTERED_X2_SUM_OPCODE)
 			return;
 		if (effective_len <= 1) {
 			res->outbuf.one_double[0] = NA_REAL;
@@ -1285,7 +1346,7 @@ static inline int is_single_NA(SEXP x)
 			return 1;
 	    break;
 	    case CPLXSXP:
-		if (LENGTH(x) == 1 && RCOMPLEX_IS_NA(COMPLEX(x)))
+		if (LENGTH(x) == 1 && RCOMPLEX_IS_NA_OR_NaN(COMPLEX(x)))
 			return 1;
 	    break;
 	    case STRSXP:
