@@ -6,6 +6,10 @@
 
 #include <string.h>  /* for memset() and memcpy() */
 
+/* Initialized in R_init_SparseArray(). */
+int intNA;
+double doubleNA;
+Rcomplex RcomplexNA;
 
 /* The 7 types of R vectors (6 types of atomic vectors + the "list" type). */
 static const SEXPTYPE Rvector_types[] = {
@@ -57,16 +61,19 @@ size_t _get_Rtype_size(SEXPTYPE Rtype)
  * _set_elts_to_zero()
  * _set_elts_to_one()
  * _set_elts_to_minus_one()
+ * _set_elts_to_NA()
  *
  * _set_Rsubvec_elts_to_val()
  * _set_Rsubvec_elts_to_zero()
  * _set_Rsubvec_elts_to_one()
  * _set_Rsubvec_elts_to_minus_one()
+ * _set_Rsubvec_elts_to_NA()
  *
  * _set_Rvector_elts_to_val()
  * _set_Rvector_elts_to_zero()
  * _set_Rvector_elts_to_one()
  * _set_Rvector_elts_to_minus_one()
+ * _set_Rvector_elts_to_NA()
  */
 
 #define	DEFINE_set_TYPE_elts_to_val_FUN(type)				\
@@ -172,6 +179,34 @@ void _set_elts_to_minus_one(SEXPTYPE Rtype, void *x,
 	return;
 }
 
+/* Restricted to types "integer", "logical", "double", and "complex". */
+void _set_elts_to_NA(SEXPTYPE Rtype, void *x, R_xlen_t offset, R_xlen_t n)
+{
+	switch (Rtype) {
+	    case INTSXP: {
+		_set_elts_to_val(Rtype, x, offset, n, &intNA);
+		return;
+	    }
+	    case LGLSXP: {
+		const int val = NA_LOGICAL;
+		_set_elts_to_val(Rtype, x, offset, n, &val);
+		return;
+	    }
+	    case REALSXP: {
+		_set_elts_to_val(Rtype, x, offset, n, &doubleNA);
+		return;
+	    }
+	    case CPLXSXP: {
+		_set_elts_to_val(Rtype, x, offset, n, &RcomplexNA);
+		return;
+	    }
+	}
+	error("SparseArray internal error in _set_elts_to_NA():\n"
+	      "    type \"%s\" is not supported", type2char(Rtype));
+	return;
+}
+
+
 static void set_character_elts_to_val(SEXP Rvector,
 		R_xlen_t subvec_offset, int subvec_len, const SEXP val)
 {
@@ -224,8 +259,7 @@ void _set_Rsubvec_elts_to_zero(SEXP Rvector,
 				     R_NilValue);
 		return;
 	}
-	_set_elts_to_zero(TYPEOF(Rvector), DATAPTR(Rvector),
-			  subvec_offset, subvec_len);
+	_set_elts_to_zero(Rtype, DATAPTR(Rvector), subvec_offset, subvec_len);
 	return;
 }
 
@@ -247,6 +281,21 @@ void _set_Rsubvec_elts_to_minus_one(SEXP Rvector,
 	return;
 }
 
+/* Restricted to types "integer", "logical", "double", "complex",
+   and "character". */
+void _set_Rsubvec_elts_to_NA(SEXP Rvector,
+		R_xlen_t subvec_offset, R_xlen_t subvec_len)
+{
+	SEXPTYPE Rtype = TYPEOF(Rvector);
+	if (Rtype == STRSXP) {
+		set_character_elts_to_val(Rvector, subvec_offset, subvec_len,
+					  NA_STRING);
+		return;
+	}
+	_set_elts_to_NA(Rtype, DATAPTR(Rvector), subvec_offset, subvec_len);
+	return;
+}
+
 /* When 'Rtype' is STRSXP or VECSXP, 'val' must be an SEXP. Otherwise, it must
    be a pointer to an int, double, Rcomplex, or Rbyte. */
 void _set_Rvector_elts_to_val(SEXP Rvector, const void *val)
@@ -275,6 +324,14 @@ void _set_Rvector_elts_to_minus_one(SEXP Rvector)
 	return;
 }
 
+/* Restricted to types "integer", "logical", "double", "complex",
+   and "character". */
+void _set_Rvector_elts_to_NA(SEXP Rvector)
+{
+	_set_Rsubvec_elts_to_NA(Rvector, 0, XLENGTH(Rvector));
+	return;
+}
+
 
 /****************************************************************************
  * _set_selected_elts_to_zero()
@@ -407,13 +464,15 @@ void _set_selected_Rsubvec_elts_to_one(SEXP Rvector, R_xlen_t subvec_offset,
  * _new_Rmatrix0()
  * _new_Rarray0()
  * _new_Rvector1()
+ * _new_RvectorNA()
+ * _new_RarrayNA()
  */
 
 /* Like allocVector() but with initialization of the vector elements. */
 SEXP _new_Rvector0(SEXPTYPE Rtype, R_xlen_t len)
 {
 	SEXP ans = PROTECT(allocVector(Rtype, len));
-	/* allocVector() does NOT initialize the vector elements, except
+	/* allocVector() does NOT initialize the vector elements, except for
 	   a character vector or a list. */
 	if (Rtype != STRSXP && Rtype != VECSXP)
 		_set_Rvector_elts_to_zero(ans);
@@ -463,9 +522,35 @@ SEXP _new_Rvector1(SEXPTYPE Rtype, int len)
 	return ans;
 }
 
+/* Like _new_Rvector0() but:
+   - initializes the array elements to NA;
+   - restricted to types "integer", "logical", "double", "complex",
+     and "character". */
+SEXP _new_RvectorNA(SEXPTYPE Rtype, R_xlen_t len)
+{
+	SEXP ans = PROTECT(allocVector(Rtype, len));
+	_set_Rvector_elts_to_NA(ans);
+	UNPROTECT(1);
+	return ans;
+}
+
+/* Like _new_Rarray0() but:
+   - initializes the array elements to NA;
+   - restricted to types "integer", "logical", "double", "complex",
+     and "character". */
+SEXP _new_RarrayNA(SEXPTYPE Rtype, SEXP dim, SEXP dimnames)
+{
+	SEXP ans = PROTECT(allocArray(Rtype, dim));
+	_set_Rvector_elts_to_NA(ans);
+	SET_DIMNAMES(ans, dimnames);
+	UNPROTECT(1);
+	return ans;
+}
+
 
 /****************************************************************************
  * _collect_offsets_of_nonzero_Rsubvec_elts()
+ * _collect_offsets_of_nonNA_Rsubvec_elts()
  */
 
 static int collect_offsets_of_nonzero_int_elts(
@@ -478,6 +563,16 @@ static int collect_offsets_of_nonzero_int_elts(
 	return (int) (out - out0);
 }
 
+static int collect_offsets_of_nonNA_int_elts(
+		const int *x, int n, int *out)
+{
+	const int *out0 = out;
+	for (int i = 0; i < n; i++)
+		if (x[i] != NA_INTEGER)
+			*(out++) = i;
+	return (int) (out - out0);
+}
+
 static int collect_offsets_of_nonzero_double_elts(
 		const double *x, int n, int *out)
 {
@@ -488,13 +583,33 @@ static int collect_offsets_of_nonzero_double_elts(
 	return (int) (out - out0);
 }
 
-#define	IS_NONZERO_RCOMPLEX(x) ((x)->r != double0 || (x)->i != double0)
+static int collect_offsets_of_nonNA_double_elts(
+		const double *x, int n, int *out)
+{
+	const int *out0 = out;
+	for (int i = 0; i < n; i++)
+		if (!R_IsNA(x[i]))  // do NOT use ISNAN()!
+			*(out++) = i;
+	return (int) (out - out0);
+}
+
 static int collect_offsets_of_nonzero_Rcomplex_elts(
 		const Rcomplex *x, int n, int *out)
 {
 	const int *out0 = out;
 	for (int i = 0; i < n; i++, x++) {
-		if (IS_NONZERO_RCOMPLEX(x))
+		if (x->r != Rcomplex0.r || x->i != Rcomplex0.i)
+			*(out++) = i;
+	}
+	return (int) (out - out0);
+}
+
+static int collect_offsets_of_nonNA_Rcomplex_elts(
+		const Rcomplex *x, int n, int *out)
+{
+	const int *out0 = out;
+	for (int i = 0; i < n; i++, x++) {
+		if (!(R_IsNA(x->r) || R_IsNA(x->i)))
 			*(out++) = i;
 	}
 	return (int) (out - out0);
@@ -510,15 +625,27 @@ static int collect_offsets_of_nonzero_Rbyte_elts(
 	return (int) (out - out0);
 }
 
-#define	IS_NONEMPTY_CHARSXP(x) ((x) == NA_STRING || XLENGTH(x) != 0)
 static int collect_offsets_of_nonempty_character_elts(
 		SEXP Rvector, R_xlen_t subvec_offset, int subvec_len,
 		int *out)
 {
 	const int *out0 = out;
 	for (int i = 0; i < subvec_len; i++, subvec_offset++) {
-		SEXP elt = STRING_ELT(Rvector, subvec_offset);
-		if (IS_NONEMPTY_CHARSXP(elt))
+		SEXP vec_elt = STRING_ELT(Rvector, subvec_offset);
+		if (!IS_EMPTY_CHARSXP(vec_elt))
+			*(out++) = i;
+	}
+	return (int) (out - out0);
+}
+
+static int collect_offsets_of_nonNA_character_elts(
+		SEXP Rvector, R_xlen_t subvec_offset, int subvec_len,
+		int *out)
+{
+	const int *out0 = out;
+	for (int i = 0; i < subvec_len; i++, subvec_offset++) {
+		SEXP vec_elt = STRING_ELT(Rvector, subvec_offset);
+		if (vec_elt != NA_STRING)
 			*(out++) = i;
 	}
 	return (int) (out - out0);
@@ -530,8 +657,8 @@ static int collect_offsets_of_nonnull_list_elts(
 {
 	const int *out0 = out;
 	for (int i = 0; i < subvec_len; i++, subvec_offset++) {
-		SEXP elt = VECTOR_ELT(Rvector, subvec_offset);
-		if (elt != R_NilValue)
+		SEXP vec_elt = VECTOR_ELT(Rvector, subvec_offset);
+		if (vec_elt != R_NilValue)
 			*(out++) = i;
 	}
 	return (int) (out - out0);
@@ -584,6 +711,38 @@ int _collect_offsets_of_nonzero_Rsubvec_elts(
 	return -1;  /* will never reach this */
 }
 
+/* Similar to _collect_offsets_of_nonzero_Rsubvec_elts() above but collects
+   non-NAs instead of nonzeros. Restricted to types "integer", "logical",
+   "double", "complex", and "character". */
+int _collect_offsets_of_nonNA_Rsubvec_elts(
+		SEXP Rvector, R_xlen_t subvec_offset, int subvec_len,
+		int *out)
+{
+	SEXPTYPE Rtype = TYPEOF(Rvector);
+	switch (Rtype) {
+	    case INTSXP: case LGLSXP:
+		return collect_offsets_of_nonNA_int_elts(
+				INTEGER(Rvector) + subvec_offset,
+				subvec_len, out);
+	    case REALSXP:
+		return collect_offsets_of_nonNA_double_elts(
+				REAL(Rvector) + subvec_offset,
+				subvec_len, out);
+	    case CPLXSXP:
+		return collect_offsets_of_nonNA_Rcomplex_elts(
+				COMPLEX(Rvector) + subvec_offset,
+				subvec_len, out);
+	    case STRSXP:
+		return collect_offsets_of_nonNA_character_elts(
+				Rvector, subvec_offset,
+				subvec_len, out);
+	}
+	error("SparseArray internal error in "
+	      "_collect_offsets_of_nonNA_Rsubvec_elts():\n"
+	      "    type \"%s\" is not supported", type2char(Rtype));
+	return -1;  /* will never reach this */
+}
+
 
 /****************************************************************************
  * _all_elts_equal_one()
@@ -754,7 +913,7 @@ int _all_selected_Rsubvec_elts_equal_one(SEXP Rvector, R_xlen_t subvec_offset,
  * _select_copy_Rvector_elt_FUN()
  */
 
-CopyRVectorElt_FUNType _select_copy_Rvector_elt_FUN(SEXPTYPE Rtype)
+CopyRVectorEltFUN _select_copy_Rvector_elt_FUN(SEXPTYPE Rtype)
 {
 	switch (Rtype) {
 	    case INTSXP: case LGLSXP: return copy_INTEGER_elt;
