@@ -17,7 +17,18 @@ static const double double1 = 1.0;
 static const Rcomplex Rcomplex1 = {{double1, double0}}; */
 static const Rcomplex Rcomplex1 = {{1.0, 0.0}};
 
-typedef void (*CopyRVectorElt_FUNType)(
+/* Unfortunately, R does not define NA_INTEGER or NA_REAL as const
+   variables so we can't define intNA, doubleNA, or RcomplexNA as
+   we do for int0/1, double0/1, or Rcomplex0/1 above. */
+extern int intNA;
+extern double doubleNA;
+extern Rcomplex RcomplexNA;
+
+#define	IS_EMPTY_CHARSXP(x) ((x) != NA_STRING && isBlankString(CHAR(x)))
+
+#define RCOMPLEX_IS_NA_OR_NaN(z) (ISNAN((z)->r) || ISNAN((z)->i))
+
+typedef void (*CopyRVectorEltFUN)(
 	SEXP in,  R_xlen_t in_offset,
 	SEXP out, R_xlen_t out_offset);
 
@@ -142,6 +153,13 @@ void _set_elts_to_minus_one(
 	R_xlen_t n
 );
 
+void _set_elts_to_NA(
+	SEXPTYPE Rtype,
+	void *x,
+	R_xlen_t offset,
+	R_xlen_t n
+);
+
 void _set_Rsubvec_elts_to_val(
 	SEXP Rvector,
 	R_xlen_t subvec_offset,
@@ -167,6 +185,12 @@ void _set_Rsubvec_elts_to_minus_one(
 	R_xlen_t subvec_len
 );
 
+void _set_Rsubvec_elts_to_NA(
+	SEXP Rvector,
+	R_xlen_t subvec_offset,
+	R_xlen_t subvec_len
+);
+
 void _set_Rvector_elts_to_val(SEXP Rvector, const void *val);
 
 void _set_Rvector_elts_to_zero(SEXP Rvector);
@@ -175,6 +199,8 @@ void _set_Rvector_elts_to_one(SEXP Rvector);
 
 void _set_Rvector_elts_to_minus_one(SEXP Rvector);
 
+void _set_Rvector_elts_to_NA(SEXP Rvector);
+
 void _set_selected_elts_to_zero(
 	SEXPTYPE Rtype,
 	void *x,
@@ -228,6 +254,17 @@ SEXP _new_Rvector1(
 	int len
 );
 
+SEXP _new_RvectorNA(
+	SEXPTYPE Rtype,
+	R_xlen_t len
+);
+
+SEXP _new_RarrayNA(
+	SEXPTYPE Rtype,
+	SEXP dim,
+	SEXP dimnames
+);
+
 int _collect_offsets_of_nonzero_Rsubvec_elts(
 	SEXP Rvector,
 	R_xlen_t subvec_offset,
@@ -235,6 +272,13 @@ int _collect_offsets_of_nonzero_Rsubvec_elts(
 	int *out
 );
 
+int _collect_offsets_of_nonNA_Rsubvec_elts(
+	SEXP Rvector,
+	R_xlen_t subvec_offset,
+	int subvec_len,
+	int *out
+);
+
 int _all_elts_equal_one(
 	SEXPTYPE Rtype,
 	const void *x,
@@ -256,7 +300,7 @@ int _all_selected_Rsubvec_elts_equal_one(
 	int n
 );
 
-CopyRVectorElt_FUNType _select_copy_Rvector_elt_FUN(SEXPTYPE Rtype);
+CopyRVectorEltFUN _select_copy_Rvector_elt_FUN(SEXPTYPE Rtype);
 
 void _copy_Rvector_elts(
 	SEXP in,
