@@ -12,7 +12,7 @@
  ****************************************************************************/
 #include "SparseArray_aperm.h"
 
-#include "Rvector_utils.h"
+#include "argcheck_utils.h"
 #include "leaf_utils.h"
 
 #include <string.h>  /* for memset() */
@@ -91,45 +91,17 @@ static SEXP alloc_output_leaf(SEXPTYPE Rtype, int nzcount,
 	*quick_out_nzoffs_p = INTEGER(nzoffs);
 	SEXP nzvals;
 	if (onecount_buf != NULL && *onecount_buf == nzcount) {
-		/* Create a lacunar leaf. Will turn it into a standard leaf
-		   later if LACUNAR_MODE_IS_ON is set to 0. */
+		/* Create a lacunar leaf. */
 		nzvals = R_NilValue;
 	} else {
 		nzvals = PROTECT(allocVector(Rtype, nzcount));
 	}
 	set_quick_out_nzvals_p(quick_out_nzvals_p, Rtype, nzvals);
-	SEXP ans = PROTECT(zip_leaf(nzvals, nzoffs));
+	SEXP ans = PROTECT(zip_leaf(nzvals, nzoffs, 0));
 	UNPROTECT(nzvals == R_NilValue ? 2 : 3);
 	return ans;
 }
 
-/* Inplace replacement!
-   Maybe move this to SVT_SparseArray_class.c. */
-static void REC_replace_lacunar_leaves_with_standard_leaves(
-		SEXP SVT, const int *dim, int ndim, SEXPTYPE Rtype)
-{
-	if (SVT == R_NilValue)
-		return;
-	if (ndim == 1) {
-		/* 'SVT' is a leaf (i.e. a 1D SVT). */
-		SEXP nzvals, nzoffs;
-		int nzcount = unzip_leaf(SVT, &nzvals, &nzoffs);
-		if (nzvals != R_NilValue)  /* standard leaf */
-			return;
-		/* lacunar leaf */
-		nzvals = PROTECT(_new_Rvector1(Rtype, nzcount));
-		replace_leaf_nzvals(SVT, nzvals);
-		UNPROTECT(1);
-		return;
-	}
-	/* 'SVT' is a tree. */
-	int SVT_len = LENGTH(SVT);  /* same as dim[ndim - 1] */
-	for (int i = 0; i < SVT_len; i++)
-		REC_replace_lacunar_leaves_with_standard_leaves(
-			VECTOR_ELT(SVT, i), dim, ndim - 1, Rtype);
-	return;
-}
-
 
 /****************************************************************************
  *									    *
@@ -170,7 +142,7 @@ static void collect_stats_on_input_rows(SEXP SVT, int nrow, int ncol,
 	return;
 }
 
-typedef void (*TransposeCol_FUNType)(int col_idx, SEXP col,
+typedef void (*TransposeColFUN)(int col_idx, SEXP col,
 		void **quick_out_nzvals_p, int **quick_out_nzoffs_p,
 		int *nzcount_buf);
 
@@ -332,7 +304,7 @@ static void transpose_list_col(int col_idx, SEXP col,
 	return;
 }
 
-static TransposeCol_FUNType select_transpose_col_FUN(SEXPTYPE Rtype)
+static TransposeColFUN select_transpose_col_FUN(SEXPTYPE Rtype)
 {
 	switch (Rtype) {
 	    case INTSXP: case LGLSXP: return transpose_integer_col;
@@ -348,7 +320,7 @@ static TransposeCol_FUNType select_transpose_col_FUN(SEXPTYPE Rtype)
 static SEXP transpose_2D_SVT(SEXP SVT, int nrow, int ncol, SEXPTYPE Rtype,
 		int *nzcount_buf, int *onecount_buf)
 {
-	TransposeCol_FUNType transpose_col_FUN =
+	TransposeColFUN transpose_col_FUN =
 			select_transpose_col_FUN(Rtype);
 	if (transpose_col_FUN == NULL)
 		error("SparseArray internal error in "
@@ -389,13 +361,6 @@ static SEXP transpose_2D_SVT(SEXP SVT, int nrow, int ncol, SEXPTYPE Rtype,
 			nzcount_buf);
 	}
 
-	/* 4th pass */
-	if (onecount_buf != NULL && LACUNAR_MODE_IS_ON == 0) {
-		const int ans_dim[2] = {ncol, nrow};
-		REC_replace_lacunar_leaves_with_standard_leaves(
-			ans, ans_dim, 2, Rtype);
-	}
-
 	UNPROTECT(1);
 	return ans;
 }
@@ -403,11 +368,8 @@ static SEXP transpose_2D_SVT(SEXP SVT, int nrow, int ncol, SEXPTYPE Rtype,
 /* --- .Call ENTRY POINT --- */
 SEXP C_transpose_2D_SVT(SEXP x_dim, SEXP x_type, SEXP x_SVT)
 {
-	SEXPTYPE x_Rtype = _get_Rtype_from_Rstring(x_type);
-	if (x_Rtype == 0)
-		error("SparseArray internal error in "
-		      "C_transpose_2D_SVT():\n"
-		      "    SVT_SparseMatrix object has invalid type");
+	SEXPTYPE x_Rtype = _get_and_check_Rtype_from_Rstring(x_type,
+					"C_transpose_2D_SVT", "x_type");
 
 	if (LENGTH(x_dim) != 2)
 		error("object to transpose must have exactly 2 dimensions");
@@ -922,11 +884,6 @@ static SEXP aperm_SVT_shattering_leaves(
 				A0Bufs->quick_out_nzvals_p,
 				A0Bufs->quick_out_nzoffs_p);
 
-	/* 4th pass */
-	if (A0Bufs->onecount_buf != NULL && LACUNAR_MODE_IS_ON == 0)
-		REC_replace_lacunar_leaves_with_standard_leaves(
-			ans, ans_dim, ndim, Rtype);
-
 	UNPROTECT(1);
 	return ans;
 }
@@ -972,10 +929,8 @@ static SEXP aperm0_SVT(SEXP SVT, const int *dim, int ndim, SEXPTYPE Rtype,
  */
 SEXP C_aperm0_SVT(SEXP x_dim, SEXP x_type, SEXP x_SVT, SEXP perm)
 {
-	SEXPTYPE Rtype = _get_Rtype_from_Rstring(x_type);
-	if (Rtype == 0)
-		error("SparseArray internal error in C_aperm0_SVT():\n"
-		      "    SVT_SparseArray object has invalid type");
+	SEXPTYPE Rtype = _get_and_check_Rtype_from_Rstring(x_type,
+						"C_aperm0_SVT", "x_type");
 
 	int ndim = LENGTH(x_dim);
 	const int *dim = INTEGER(x_dim);
@@ -1040,10 +995,8 @@ static SEXP REC_aperm_SVT(SEXP SVT, const int *dim, int ndim, SEXPTYPE Rtype,
  */
 SEXP C_aperm_SVT(SEXP x_dim, SEXP x_type, SEXP x_SVT, SEXP perm)
 {
-	SEXPTYPE Rtype = _get_Rtype_from_Rstring(x_type);
-	if (Rtype == 0)
-		error("SparseArray internal error in C_aperm_SVT():\n"
-		      "    SVT_SparseArray object has invalid type");
+	SEXPTYPE Rtype = _get_and_check_Rtype_from_Rstring(x_type,
+						"C_aperm_SVT", "x_type");
 
 	int ndim = LENGTH(x_dim);
 	const int *dim = INTEGER(x_dim);
