@@ -3,9 +3,14 @@
 
 #include <Rdefines.h>
 
-/* Copied from S4Arrays/src/array_selection.h */
-#define	INVALID_COORD(coord, maxcoord) \
-	((coord) == NA_INTEGER || (coord) < 1 || (coord) > (maxcoord))
+SEXP C_subassign_SVT_by_Lindex(
+	SEXP x_dim,
+	SEXP x_type,
+	SEXP x_SVT,
+	SEXP x_na_background,
+	SEXP Lindex,
+	SEXP vals
+);
 
 SEXP C_subassign_SVT_by_Mindex(
 	SEXP x_dim,
@@ -15,19 +20,11 @@ SEXP C_subassign_SVT_by_Mindex(
 	SEXP vals
 );
 
-SEXP C_subassign_SVT_by_Lindex(
-	SEXP x_dim,
-	SEXP x_type,
-	SEXP x_SVT,
-	SEXP Lindex,
-	SEXP vals
-);
-
 SEXP C_subassign_SVT_with_short_Rvector(
 	SEXP x_dim,
 	SEXP x_type,
 	SEXP x_SVT,
-	SEXP index,
+	SEXP Nindex,
 	SEXP Rvector
 );
 
@@ -35,7 +32,7 @@ SEXP C_subassign_SVT_with_Rarray(
 	SEXP x_dim,
 	SEXP x_type,
 	SEXP x_SVT,
-	SEXP index,
+	SEXP Nindex,
 	SEXP Rarray
 );
 
@@ -43,7 +40,7 @@ SEXP C_subassign_SVT_with_SVT(
 	SEXP x_dim,
 	SEXP x_type,
 	SEXP x_SVT,
-	SEXP index,
+	SEXP Nindex,
 	SEXP v_dim,
 	SEXP v_type,
 	SEXP v_SVT
