@@ -7,9 +7,9 @@
  ****************************************************************************/
 #include "SparseArray_subsetting.h"
 
+#include "argcheck_utils.h"
 #include "OPBufTree.h"
 #include "thread_control.h"  /* for which_max() */
-#include "Rvector_utils.h"
 #include "leaf_utils.h"
 
 #include <limits.h>  /* for INT_MAX */
@@ -52,122 +52,6 @@ static inline void set_Rvector_elt_to_NA(SEXP Rvector, R_xlen_t i)
 }
 
 
-/****************************************************************************
- * extract_long_idx0() and extract_idx0()
- */
-
-/* In addition to 0, extract_long_idx0() and extract_idx0() can return one
-   of the five negative values defined below. Note that these are typically
-   considered errors but not always. For example, some functions in this file
-   treat SUBSCRIPT_ELT_IS_NA as an error but others don't.
-   Be aware that MAX_OPBUF_LEN_REACHED is set to -1 (see OPBufTree.h) so
-   do **not** use that value. */
-#define	BAD_SUBSCRIPT_TYPE             -2
-#define	SUBSCRIPT_IS_TOO_LONG          -3  /* returned by extract_idx0() only */
-#define	SUBSCRIPT_ELT_IS_LESS_THAN_ONE -4
-#define	SUBSCRIPT_ELT_IS_BEYOND_MAX    -5
-#define	SUBSCRIPT_ELT_IS_NA            -6
-
-/* 'subscript' must be a numeric vector, possibly a long one. It is expected
-   to contain 1-based indices that are >= 1 and <= 'max'.
-   Returns 0 or one of the values defined above. Will set 'idx0' only if
-   returning 0. In other words, caller **must** ignore 'idx0' when a non-zero
-   value is returned. */
-static inline int extract_long_idx0(SEXP subscript, R_xlen_t i,
-				    R_xlen_t max, R_xlen_t *idx0)
-{
-	if (IS_INTEGER(subscript)) {
-		int idx = INTEGER(subscript)[i];
-		if (idx == NA_INTEGER)
-			return SUBSCRIPT_ELT_IS_NA;
-		idx--;	/* from 1-based to 0-based */
-		if (idx < 0)
-			return SUBSCRIPT_ELT_IS_LESS_THAN_ONE;
-		if ((R_xlen_t) idx >= max)  /* >= yes! */
-			return SUBSCRIPT_ELT_IS_BEYOND_MAX;
-		*idx0 = (R_xlen_t) idx;
-		return 0;
-	}
-	if (IS_NUMERIC(subscript)) {
-		double idx = REAL(subscript)[i];
-		/* ISNAN(): True for *both* NA and NaN. See <R_ext/Arith.h> */
-		if (ISNAN(idx))
-			return SUBSCRIPT_ELT_IS_NA;
-		idx -= 1.0;  /* from 1-based to 0-based */
-		if (idx < 0.0)
-			return SUBSCRIPT_ELT_IS_LESS_THAN_ONE;
-		if (idx >= (double) max)  /* >= yes! */
-			return SUBSCRIPT_ELT_IS_BEYOND_MAX;
-		*idx0 = (R_xlen_t) idx;
-		return 0;
-	}
-	return BAD_SUBSCRIPT_TYPE;
-}
-
-/* Like extract_long_idx0(), except that 'subscript' cannot be a long vector
-   nor can it contain values > INT_MAX ('max' must be supplied as an 'int'). */
-static inline int extract_idx0(SEXP subscript, int i, int max, int *idx0)
-{
-	if (XLENGTH(subscript) > (R_xlen_t) INT_MAX)
-		return SUBSCRIPT_IS_TOO_LONG;
-	R_xlen_t lidx0;
-	int ret = extract_long_idx0(subscript, (R_xlen_t) i,
-				    (R_xlen_t) max, &lidx0);
-	if (ret < 0)
-		return ret;
-	*idx0 = (int) lidx0;
-	return 0;
-}
-
-/* 'ret_code' must be a negative code (error code) as returned by
-   extract_long_idx0() or extract_idx0() above. */
-static void bad_Lindex_error(int ret_code)
-{
-	if (ret_code == BAD_SUBSCRIPT_TYPE)
-		error("linear index (L-index) must be a numeric vector");
-	if (ret_code == SUBSCRIPT_IS_TOO_LONG)
-		error("linear index (L-index) is too long");
-	if (ret_code == SUBSCRIPT_ELT_IS_LESS_THAN_ONE ||
-	    ret_code == SUBSCRIPT_ELT_IS_BEYOND_MAX)
-		error("linear index (L-index) contains out-of-bound indices");
-	if (ret_code == MAX_OPBUF_LEN_REACHED)
-		error("too many indices in the linear index (L-index) hit the "
-		      "same leaf in the Sparse Vector Tree representation");
-	error("SparseArray internal error in bad_Lindex_error():\n"
-	      "    unexpected error code %d", ret_code);
-}
-
-/* 'ret_code' must be a negative code (error code) as returned by
-   extract_long_idx0() or extract_idx0() above. */
-static void bad_Mindex_error(int ret_code)
-{
-	if (ret_code == BAD_SUBSCRIPT_TYPE)
-		error("matrix subscript (M-index) must be a numeric matrix");
-	if (ret_code == SUBSCRIPT_ELT_IS_LESS_THAN_ONE ||
-	    ret_code == SUBSCRIPT_ELT_IS_BEYOND_MAX)
-		error("matrix subscript (M-index) contains "
-		      "out-of-bound indices");
-	if (ret_code == SUBSCRIPT_ELT_IS_NA)
-		error("matrix subscript (M-index) contains NAs");
-	error("SparseArray internal error in bad_Mindex_error():\n"
-	      "    unexpected error code %d", ret_code);
-}
-
-/* 'ret_code' must be a negative code (error code) as returned by
-   extract_idx0() above. */
-static void bad_Nindex_error(int ret_code, int along1)
-{
-	if (ret_code == BAD_SUBSCRIPT_TYPE)
-		error("'Nindex[[%d]]' is not a numeric vector (or a NULL)",
-		      along1);
-	if (ret_code == SUBSCRIPT_IS_TOO_LONG)
-		error("'Nindex[[%d]]' is too long", along1);
-	if (ret_code == SUBSCRIPT_ELT_IS_NA)
-		error("'Nindex[[%d]]' contains NAs", along1);
-	error("'Nindex[[%d]]' contains out-of-bound indices", along1);
-}
-
-
 /****************************************************************************
  * subset_NULL_by_Lindex()
  * subset_leaf_by_Lindex()
@@ -262,7 +146,7 @@ static int subset_NULL_by_Lindex(int dim0, SEXP Lindex, SEXP ans)
    1D SVT_SparseArray object, 'dim0' is also the length of the object).
    NA indices are ok. */
 static int subset_leaf_by_Lindex(SEXP leaf, int dim0, SEXP Lindex, SEXP ans,
-		CopyRVectorElt_FUNType copy_Rvector_elt_FUN)
+		CopyRVectorEltFUN copy_Rvector_elt_FUN)
 {
 	if (leaf == R_NilValue)
 		return subset_NULL_by_Lindex(dim0, Lindex, ans);
@@ -292,7 +176,7 @@ static int subset_leaf_by_Lindex(SEXP leaf, int dim0, SEXP Lindex, SEXP ans,
 }
 
 static void subset_leaf_by_OPBuf(SEXP leaf, const OPBuf *opbuf, SEXP ans,
-		CopyRVectorElt_FUNType copy_Rvector_elt_FUN, int *lookup_table)
+		CopyRVectorEltFUN copy_Rvector_elt_FUN, int *lookup_table)
 {
 	if (leaf == R_NilValue || opbuf->nelt == 0)
 		return;
@@ -351,11 +235,11 @@ static int subset_SV(const SparseVec *sv, SEXP subscript,
 	if (use_lookup_table)
 		build_lookup_table(lookup_table, sv->nzoffs, sv_nzcount);
 	/* Walk on 'subscript'. */
+	int idx0 = 0;  // only for -Wmaybe-uninitialized
 	for (int i1 = 0; i1 < n; i1++) {
-		int idx0;
 		int ret = extract_idx0(subscript, i1, sv->len, &idx0);
 		if (ret < 0)
-			bad_Nindex_error(ret, 1);
+			_bad_Nindex_error(ret, 1);
 		int k2 = MAP_IDX0_TO_K(idx0, sv->nzoffs, sv_nzcount);
 		if (k2 >= 0) {
 			sv_selection[out_nzcount] = k2;
@@ -380,7 +264,7 @@ static SEXP subset_leaf_as_sparse(SEXP leaf, int dim0, SEXP subscript,
 		return leaf;
 
 	SEXP leaf_nzvals = get_leaf_nzvals(leaf);
-	SparseVec sv = leaf2SV(leaf, TYPEOF(leaf_nzvals), dim0);
+	SparseVec sv = leaf2SV(leaf, TYPEOF(leaf_nzvals), dim0, 0);
 	int ans_nzcount = subset_SV(&sv, subscript,
 				    selection_buf, nzoffs_buf, lookup_table);
 	if (ans_nzcount == 0)
@@ -393,43 +277,32 @@ static SEXP subset_leaf_as_sparse(SEXP leaf, int dim0, SEXP subscript,
 		UNPROTECT(1);
 		return ans;
 	}
-	/* input leaf is standard */
-	if (LACUNAR_MODE_IS_ON) {
-		int all_ones = _all_selected_Rsubvec_elts_equal_one(
-						leaf_nzvals, 0,
-						selection_buf, ans_nzcount);
-		if (all_ones) {
-			SEXP ans = _make_lacunar_leaf(ans_nzoffs);
-			UNPROTECT(1);
-			return ans;
-		}
+	/* Input leaf is standard */
+	int all_ones = _all_selected_Rsubvec_elts_equal_one(
+					     leaf_nzvals, 0,
+					     selection_buf, ans_nzcount);
+	if (all_ones) {
+		SEXP ans = _make_lacunar_leaf(ans_nzoffs);
+		UNPROTECT(1);
+		return ans;
 	}
 	SEXP ans_nzvals = PROTECT(
 		_subset_Rsubvec(leaf_nzvals, 0, selection_buf, ans_nzcount)
 	);
-	SEXP ans = zip_leaf(ans_nzvals, ans_nzoffs);
+	SEXP ans = zip_leaf(ans_nzvals, ans_nzoffs, 0);
 	UNPROTECT(2);
 	return ans;
 }
 
 
 /****************************************************************************
- * C_subset_SVT_by_[L|M]index()
+ * build_OPBufTree_from_Lindex()
+ * build_OPBufTree_from_Mindex()
+ *
+ * Both are used by .Call ENTRY POINTs C_subset_SVT_by_[L|M]index() below
+ * in this file.
  */
 
-static int check_Mindex(SEXP Mindex, int ndim,
-		const char *what1, const char *what2)
-{
-	SEXP Mindex_dim = GET_DIM(Mindex);
-	if (Mindex_dim == R_NilValue || LENGTH(Mindex_dim) != 2)
-		error("'%s' must be a matrix", what1);
-	if (!(IS_INTEGER(Mindex) || IS_NUMERIC(Mindex)))
-		error("'%s' must be an integer matrix", what1);
-	if (INTEGER(Mindex_dim)[1] != ndim)
-		error("ncol(%s) != %s", what1, what2);
-	return INTEGER(Mindex_dim)[0];
-}
-
 /* 'Lidx0' is trusted to be a non-NA value >= 0 and < 'dimcumprod[ndim - 1]'.
    'SVT' is trusted to not be NULL.
    Returns NULL if we didn't land anywhere. */
@@ -610,9 +483,9 @@ static int build_OPBufTree_from_Lindex(OPBufTree *opbuf_tree, SEXP Lindex,
 	_free_OPBufTree(opbuf_tree);
 	return XLENGTH(Lindex) <= (R_xlen_t) INT_MAX ?
 		build_OPBufTree_from_Lindex1(opbuf_tree, Lindex,
-			x_SVT, x_dim, x_ndim, ans, dimcumprod) :
+				x_SVT, x_dim, x_ndim, ans, dimcumprod) :
 		build_OPBufTree_from_Lindex2(opbuf_tree, Lindex,
-			x_SVT, x_dim, x_ndim, ans, dimcumprod);
+				x_SVT, x_dim, x_ndim, ans, dimcumprod);
 }
 
 static int build_OPBufTree_from_Mindex(OPBufTree *opbuf_tree, SEXP Mindex,
@@ -651,11 +524,23 @@ static int build_OPBufTree_from_Mindex(OPBufTree *opbuf_tree, SEXP Mindex,
 	return max_outleaf_len;
 }
 
+
+/****************************************************************************
+ * C_subset_SVT_by_[L|M]index()
+ */
+
+static SEXP new_Rvector(SEXPTYPE Rtype, R_xlen_t len, int na_background)
+{
+	if (na_background)
+		return _new_RvectorNA(Rtype, len);
+	return _new_Rvector0(Rtype, len);
+}
+
 /* Recursive tree traversal must be guided by 'opbuf_tree', not by 'SVT'. See
-   long comment above why. */
+   IMPORTANT comment for build_OPBufTree_from_Lindex() above. */
 static void REC_subset_SVT_by_OPBufTree(OPBufTree *opbuf_tree,
 		SEXP SVT, const int *dim, int ndim, SEXP ans,
-		CopyRVectorElt_FUNType fun, int *lookup_table, int pardim)
+		CopyRVectorEltFUN fun, int *lookup_table, int pardim)
 {
 	if (opbuf_tree->node_type == NULL_NODE)
 		return;
@@ -690,29 +575,30 @@ static void REC_subset_SVT_by_OPBufTree(OPBufTree *opbuf_tree,
 /* --- .Call ENTRY POINT ---
    'Lindex' must be a numeric vector (integer or double), possibly a long one.
    NA indices are accepted. */
-SEXP C_subset_SVT_by_Lindex(SEXP x_dim, SEXP x_type, SEXP x_SVT, SEXP Lindex)
+SEXP C_subset_SVT_by_Lindex(
+		SEXP x_dim, SEXP x_type, SEXP x_SVT, SEXP x_na_background,
+		SEXP Lindex)
 {
-	SEXPTYPE Rtype = _get_Rtype_from_Rstring(x_type);
-	if (Rtype == 0)
-		error("SparseArray internal error in "
-		      "C_subset_SVT_by_Lindex():\n"
-		      "    SVT_SparseArray object has invalid type");
-	CopyRVectorElt_FUNType fun = _select_copy_Rvector_elt_FUN(Rtype);
+	SEXPTYPE Rtype = _get_and_check_Rtype_from_Rstring(x_type,
+				"C_subset_SVT_by_Lindex", "x_type");
+	CopyRVectorEltFUN fun = _select_copy_Rvector_elt_FUN(Rtype);
+	int x_has_NAbg = _get_and_check_na_background(x_na_background,
+				"C_subset_SVT_by_Lindex", "x_na_background");
 
-	if (!IS_INTEGER(Lindex) && !IS_NUMERIC(Lindex))
+	if (!(IS_INTEGER(Lindex) || IS_NUMERIC(Lindex)))
 		error("'Lindex' must be an integer or numeric vector");
 
 	int x_ndim = LENGTH(x_dim);
 	int x_dim0 = INTEGER(x_dim)[0];
 	R_xlen_t ans_len = XLENGTH(Lindex);
-	SEXP ans = PROTECT(_new_Rvector0(Rtype, ans_len));
+	SEXP ans = PROTECT(new_Rvector(Rtype, ans_len, x_has_NAbg));
 
 	if (x_ndim == 1) {
 		int ret = subset_leaf_by_Lindex(x_SVT, x_dim0, Lindex, ans,
 						fun);
 		UNPROTECT(1);
 		if (ret < 0)
-			bad_Lindex_error(ret);
+			_bad_Lindex_error(ret);
 		return ans;
 	}
 
@@ -731,7 +617,7 @@ SEXP C_subset_SVT_by_Lindex(SEXP x_dim, SEXP x_type, SEXP x_SVT, SEXP Lindex)
 				dimcumprod);
 	if (max_outleaf_len < 0) {
 		UNPROTECT(1);
-		bad_Lindex_error(max_outleaf_len);
+		_bad_Lindex_error(max_outleaf_len);
 	}
 	if (x_SVT == R_NilValue) {
 		UNPROTECT(1);
@@ -762,30 +648,44 @@ SEXP C_subset_SVT_by_Lindex(SEXP x_dim, SEXP x_type, SEXP x_SVT, SEXP Lindex)
 	return ans;
 }
 
+static int check_Mindex(SEXP Mindex, int ndim,
+		const char *what1, const char *what2)
+{
+	SEXP Mindex_dim = GET_DIM(Mindex);
+	if (Mindex_dim == R_NilValue || LENGTH(Mindex_dim) != 2)
+		error("'%s' must be a matrix", what1);
+	if (!(IS_INTEGER(Mindex) || IS_NUMERIC(Mindex)))
+		error("'%s' must be an integer matrix", what1);
+	if (INTEGER(Mindex_dim)[1] != ndim)
+		error("ncol(%s) != %s", what1, what2);
+	return INTEGER(Mindex_dim)[0];
+}
+
 /* --- .Call ENTRY POINT ---
    'Mindex' must be a numeric matrix (integer or double) with one column per
    dimension in the array to subset. NAs in the matrix are forbidden at the
    moment (they'll trigger an error), except in the 1D case. */
-SEXP C_subset_SVT_by_Mindex(SEXP x_dim, SEXP x_type, SEXP x_SVT, SEXP Mindex)
+SEXP C_subset_SVT_by_Mindex(
+		SEXP x_dim, SEXP x_type, SEXP x_SVT, SEXP x_na_background,
+		SEXP Mindex)
 {
-	SEXPTYPE Rtype = _get_Rtype_from_Rstring(x_type);
-	if (Rtype == 0)
-		error("SparseArray internal error in "
-		      "C_subset_SVT_by_Mindex():\n"
-		      "    SVT_SparseArray object has invalid type");
-	CopyRVectorElt_FUNType fun = _select_copy_Rvector_elt_FUN(Rtype);
+	SEXPTYPE Rtype = _get_and_check_Rtype_from_Rstring(x_type,
+				"C_subset_SVT_by_Mindex", "x_type");
+	CopyRVectorEltFUN fun = _select_copy_Rvector_elt_FUN(Rtype);
+	int x_has_NAbg = _get_and_check_na_background(x_na_background,
+				"C_subset_SVT_by_Mindex", "x_na_background");
 
 	int x_ndim = LENGTH(x_dim);
 	int x_dim0 = INTEGER(x_dim)[0];
 	int ans_len = check_Mindex(Mindex, x_ndim, "Mindex", "length(dim(x))");
-	SEXP ans = PROTECT(_new_Rvector0(Rtype, (R_xlen_t) ans_len));
+	SEXP ans = PROTECT(new_Rvector(Rtype, (R_xlen_t) ans_len, x_has_NAbg));
 
 	if (x_ndim == 1) {
 		int ret = subset_leaf_by_Lindex(x_SVT, x_dim0, Mindex, ans,
 						fun);
 		UNPROTECT(1);
 		if (ret < 0)
-			bad_Mindex_error(ret);
+			_bad_Mindex_error(ret);
 		return ans;
 	}
 	if (x_SVT == R_NilValue) {
@@ -800,7 +700,7 @@ SEXP C_subset_SVT_by_Mindex(SEXP x_dim, SEXP x_type, SEXP x_SVT, SEXP Mindex)
 					    x_SVT, INTEGER(x_dim), x_ndim, ans);
 	if (max_outleaf_len < 0) {
 		UNPROTECT(1);
-		bad_Mindex_error(max_outleaf_len);
+		_bad_Mindex_error(max_outleaf_len);
 	}
 
 	/* 2nd pass: Subset SVT by OPBufTree. */
@@ -839,12 +739,12 @@ static SEXP compute_subset_dim(SEXP Nindex, SEXP x_dim)
 			continue;
 		if (!(IS_INTEGER(subscript) || IS_NUMERIC(subscript))) {
 			UNPROTECT(1);
-			bad_Nindex_error(BAD_SUBSCRIPT_TYPE, along + 1);
+			_bad_Nindex_error(BAD_SUBSCRIPT_TYPE, along + 1);
 		}
 		R_xlen_t d = XLENGTH(subscript);
 		if (d > (R_xlen_t) INT_MAX) {
 			UNPROTECT(1);
-			bad_Nindex_error(SUBSCRIPT_IS_TOO_LONG, along + 1);
+			_bad_Nindex_error(SUBSCRIPT_IS_TOO_LONG, along + 1);
 		}
 		INTEGER(ans_dim)[along] = (int) d;
 	}
@@ -881,13 +781,11 @@ static SEXP REC_subset_SVT_by_Nindex(SEXP SVT, SEXP Nindex,
 	SEXP ans = PROTECT(NEW_LIST(ans_len));
 	int is_empty = 1;
 	for (int i = 0; i < ans_len; i++) {
-		int idx0;
-		if (subscript == R_NilValue) {
-			idx0 = i;
-		} else {
+		int idx0 = i;
+		if (subscript != R_NilValue) {
 			int ret = extract_idx0(subscript, i, SVT_len, &idx0);
 			if (ret < 0)
-				bad_Nindex_error(ret, ndim);
+				_bad_Nindex_error(ret, ndim);
 		}
 		SEXP subSVT = VECTOR_ELT(SVT, idx0);
 		SEXP ans_elt = REC_subset_SVT_by_Nindex(subSVT, Nindex,
@@ -913,11 +811,9 @@ static SEXP REC_subset_SVT_by_Nindex(SEXP SVT, SEXP Nindex,
    NAs in the subscripts are forbidden (they'll trigger an error).  */
 SEXP C_subset_SVT_by_Nindex(SEXP x_dim, SEXP x_type, SEXP x_SVT, SEXP Nindex)
 {
-	SEXPTYPE Rtype = _get_Rtype_from_Rstring(x_type);
-	if (Rtype == 0)
-		error("SparseArray internal error in "
-		      "C_subset_SVT_by_Nindex():\n"
-		      "    SVT_SparseArray object has invalid type");
+	/* Returned value ignored. */
+	_get_and_check_Rtype_from_Rstring(x_type,
+					  "C_subset_SVT_by_Nindex", "x_type");
 
 	SEXP ans_dim = PROTECT(compute_subset_dim(Nindex, x_dim));
 	int ans_dim0 = INTEGER(ans_dim)[0];
