@@ -7,7 +7,7 @@
  ****************************************************************************/
 #include "SparseArray_summarization.h"
 
-#include "Rvector_utils.h"
+#include "argcheck_utils.h"
 #include "Rvector_summarization.h"
 #include "leaf_utils.h"
 
@@ -68,20 +68,17 @@ static void REC_summarize_SVT(SEXP SVT, const int *dim, int ndim,
 }
 
 static SummarizeOp replace_SummarizeOp_center_with_mean(
-		SEXP SVT, const int *dim, int ndim,
+		SEXP SVT, int na_background, const int *dim, int ndim,
 		const SummarizeOp *summarize_op)
 {
-	SummarizeOp tmp_op;
-	SummarizeResult res;
-	double SVT_mean;
-
 	/* Compute 'mean(SVT)'. */
-	tmp_op = *summarize_op;
+	SummarizeOp tmp_op = *summarize_op;
 	tmp_op.opcode = MEAN_OPCODE;
+	SummarizeResult res;
 	_init_SummarizeResult(&tmp_op, &res);
 	REC_summarize_SVT(SVT, dim, ndim, &tmp_op, &res);
-	_postprocess_SummarizeResult(&tmp_op, &res);
-	SVT_mean = res.outbuf.one_double[0];
+	_postprocess_SummarizeResult(&res, na_background, &tmp_op);
+	double SVT_mean = res.outbuf.one_double[0];
 
 	/* Set 'center' with 'mean(SVT)'. */
 	tmp_op = *summarize_op;
@@ -89,58 +86,55 @@ static SummarizeOp replace_SummarizeOp_center_with_mean(
 	return tmp_op;
 }
 
-SummarizeResult _summarize_SVT(SEXP SVT, const int *dim, int ndim,
-			       const SummarizeOp *summarize_op)
+SummarizeResult _summarize_SVT(
+		SEXP SVT, int na_background, const int *dim, int ndim,
+		const SummarizeOp *summarize_op)
 {
 	SummarizeOp tmp_op;
-	SummarizeResult res;
-
-	if (ISNAN(summarize_op->center) &&
-		(summarize_op->opcode == SUM_CENTERED_X2_OPCODE ||
-		 summarize_op->opcode == VAR1_OPCODE ||
-		 summarize_op->opcode == SD1_OPCODE))
+	if ((summarize_op->opcode == CENTERED_X2_SUM_OPCODE ||
+	     summarize_op->opcode == VAR1_OPCODE ||
+	     summarize_op->opcode == SD1_OPCODE) && ISNAN(summarize_op->center))
 	{
-		tmp_op = replace_SummarizeOp_center_with_mean(SVT, dim, ndim,
-							      summarize_op);
+		tmp_op = replace_SummarizeOp_center_with_mean(
+					SVT, na_background, dim, ndim,
+					summarize_op);
 		summarize_op = &tmp_op;
 	}
 
+	SummarizeResult res;
 	_init_SummarizeResult(summarize_op, &res);
 	REC_summarize_SVT(SVT, dim, ndim, summarize_op, &res);
-	_postprocess_SummarizeResult(summarize_op, &res);
+	_postprocess_SummarizeResult(&res, na_background, summarize_op);
 	return res;
 }
 
 /* --- .Call ENTRY POINT --- */
-SEXP C_summarize_SVT(SEXP x_dim, SEXP x_type, SEXP x_SVT,
+SEXP C_summarize_SVT(
+		SEXP x_dim, SEXP x_type, SEXP x_SVT, SEXP x_na_background,
 		SEXP op, SEXP na_rm, SEXP center)
 {
-	SEXPTYPE x_Rtype;
-	int opcode, narm;
-	SummarizeOp summarize_op;
-	SummarizeResult res;
+	SEXPTYPE x_Rtype = _get_and_check_Rtype_from_Rstring(x_type,
+					"C_summarize_SVT", "x_type");
 
-	x_Rtype = _get_Rtype_from_Rstring(x_type);
-	if (x_Rtype == 0)
-		error("SparseArray internal error in "
-		      "C_summarize_SVT():\n"
-		      "    SVT_SparseArray object has invalid type");
+	int x_has_NAbg = _get_and_check_na_background(x_na_background,
+					"C_summarize_SVT", "x_na_background");
 
-	opcode = _get_summarize_opcode(op, x_Rtype);
+	int opcode = _get_summarize_opcode(op, x_Rtype);
 
 	if (!(IS_LOGICAL(na_rm) && LENGTH(na_rm) == 1))
 		error("'na.rm' must be TRUE or FALSE");
-	narm = LOGICAL(na_rm)[0];
+	int narm = LOGICAL(na_rm)[0];
 
 	if (!IS_NUMERIC(center) || LENGTH(center) != 1)
 		error("SparseArray internal error in "
 		      "C_summarize_SVT():\n"
 		      "    'center' must be a single number");
 
-	summarize_op = _make_SummarizeOp(opcode, x_Rtype, narm,
-					 REAL(center)[0]);
-	res = _summarize_SVT(x_SVT, INTEGER(x_dim), LENGTH(x_dim),
-			     &summarize_op);
+	SummarizeOp summarize_op = _make_SummarizeOp(opcode, x_Rtype, narm,
+						     REAL(center)[0]);
+	SummarizeResult res = _summarize_SVT(x_SVT, x_has_NAbg,
+					     INTEGER(x_dim), LENGTH(x_dim),
+					     &summarize_op);
 	if (res.warn)
 		warning("NAs introduced by coercion of "
 			"infinite values to integers");
