@@ -6,6 +6,7 @@
 
 SummarizeResult _summarize_SVT(
 	SEXP SVT,
+	int na_background,
 	const int *dim,
 	int ndim,
 	const SummarizeOp *summarize_op
@@ -15,6 +16,7 @@ SEXP C_summarize_SVT(
 	SEXP x_dim,
 	SEXP x_type,
 	SEXP x_SVT,
+	SEXP x_na_background,
 	SEXP op,
 	SEXP na_rm,
 	SEXP center
