@@ -10,7 +10,7 @@
  ****************************************************************************/
 #include "SparseMatrix_mult.h"
 
-#include "Rvector_utils.h"
+#include "argcheck_utils.h"
 #include "SparseVec.h"
 #include "SparseVec_dotprod.h"
 #include "leaf_utils.h"             /* for leaf2SV() */
@@ -37,20 +37,20 @@ static int has_no_NA(const int *x, int x_len)
 
 static int doubleSV_has_no_NaN_or_Inf(const SparseVec *sv)
 {
-	if (sv->nzvals == R_NilValue)  /* lacunar SparseVec */
+	const double *nzvals_p = get_doubleSV_nzvals_p(sv);
+	if (nzvals_p == NULL)  /* lacunar SparseVec */
 		return 1;
 	/* regular SparseVec */
-	return has_no_NaN_or_Inf(get_doubleSV_nzvals_p(sv),
-				 get_SV_nzcount(sv));
+	return has_no_NaN_or_Inf(nzvals_p, get_SV_nzcount(sv));
 }
 
 static int intSV_has_no_NA(const SparseVec *sv)
 {
-	if (sv->nzvals == R_NilValue)  /* lacunar SparseVec */
+	const int *nzvals_p = get_intSV_nzvals_p(sv);
+	if (nzvals_p == NULL)  /* lacunar SparseVec */
 		return 1;
 	/* regular SparseVec */
-	return has_no_NA(get_intSV_nzvals_p(sv),
-			 get_SV_nzcount(sv));
+	return has_no_NA(nzvals_p, get_SV_nzcount(sv));
 }
 
 static void fill_col(double *out, int out_nrow, double v)
@@ -77,38 +77,12 @@ static void sym_fill_with_NAs(double *out, int out_nrow, int j)
 	return;
 }
 
-static void expand_doubleSV(const SparseVec *sv, double *out)
-{
-	memset(out, 0, sizeof(double) * sv->len);
-	if (sv->nzvals == R_NilValue) {  /* lacunar SparseVec */
-		_set_selected_elts_to_one(REALSXP, out, 0,
-				sv->nzoffs, get_SV_nzcount(sv));
-	} else {  /* regular SparseVec */
-		_copy_double_elts_to_offsets(get_doubleSV_nzvals_p(sv),
-				sv->nzoffs, get_SV_nzcount(sv), out);
-	}
-	return;
-}
-
-static void expand_intSV(const SparseVec *sv, int *out)
-{
-	memset(out, 0, sizeof(int) * sv->len);
-	if (sv->nzvals == R_NilValue) {  /* lacunar SparseVec */
-		_set_selected_elts_to_one(INTSXP, out, 0,
-				sv->nzoffs, get_SV_nzcount(sv));
-	} else {  /* regular SparseVec */
-		_copy_int_elts_to_offsets(get_intSV_nzvals_p(sv),
-				sv->nzoffs, get_SV_nzcount(sv), out);
-	}
-	return;
-}
-
 static double dotprod_leaf_finite_doubles(SEXP leaf1,
 					  const double *x2, int x2_len)
 {
 	if (leaf1 == R_NilValue)
 		return 0.0;
-	const SparseVec sv1 = leaf2SV(leaf1, REALSXP, x2_len);
+	const SparseVec sv1 = leaf2SV(leaf1, REALSXP, x2_len, 0);
 	return _dotprod_doubleSV_finite_doubles(&sv1, x2);
 }
 
@@ -117,7 +91,7 @@ static double dotprod_leaf_noNA_ints(SEXP leaf1,
 {
 	if (leaf1 == R_NilValue)
 		return 0.0;
-	const SparseVec sv1 = leaf2SV(leaf1, INTSXP, x2_len);
+	const SparseVec sv1 = leaf2SV(leaf1, INTSXP, x2_len, 0);
 	return _dotprod_intSV_noNA_ints(&sv1, x2);
 }
 
@@ -125,7 +99,7 @@ static double dotprod_leaf_doubles(SEXP leaf1, const double *x2, int x2_len)
 {
 	if (leaf1 == R_NilValue)
 		return _dotprod_doubles_zero(x2, x2_len);
-	const SparseVec sv1 = leaf2SV(leaf1, REALSXP, x2_len);
+	const SparseVec sv1 = leaf2SV(leaf1, REALSXP, x2_len, 0);
 	return _dotprod_doubleSV_doubles(&sv1, x2);
 }
 
@@ -133,7 +107,7 @@ static double dotprod_leaf_ints(SEXP leaf1, const int *x2, int x2_len)
 {
 	if (leaf1 == R_NilValue)
 		return _dotprod_ints_zero(x2, x2_len);
-	const SparseVec sv1 = leaf2SV(leaf1, INTSXP, x2_len);
+	const SparseVec sv1 = leaf2SV(leaf1, INTSXP, x2_len, 0);
 	return _dotprod_intSV_ints(&sv1, x2);
 }
 
@@ -141,7 +115,7 @@ static double dotprod_leaf_doubleSV(SEXP leaf1, const SparseVec *sv2)
 {
 	if (leaf1 == R_NilValue)
 		return _dotprod_doubleSV_zero(sv2);
-	const SparseVec sv1 = leaf2SV(leaf1, REALSXP, sv2->len);
+	const SparseVec sv1 = leaf2SV(leaf1, REALSXP, sv2->len, 0);
 	return _dotprod_doubleSV_doubleSV(&sv1, sv2);
 }
 
@@ -422,7 +396,7 @@ static void crossprod2_SVT_mat_double(SEXP SVT1, const double *mat2,
 		SEXP leaf = VECTOR_ELT(SVT1, i);
 		if (leaf == R_NilValue)
 			continue;
-		const SparseVec sv1 = leaf2SV(leaf, REALSXP, in_nrow);
+		const SparseVec sv1 = leaf2SV(leaf, REALSXP, in_nrow, 0);
 		crossprod2_doubleSV_doublemat(&sv1, REAL(mat2), in_nrow,
 					      out, out_nrow, out_ncol);
 	}
@@ -472,7 +446,7 @@ static void crossprod2_mat_SVT_double(const double *mat1, SEXP SVT2,
 		SEXP leaf = VECTOR_ELT(SVT2, j);
 		if (leaf == R_NilValue)
 			continue;
-		const SparseVec sv2 = leaf2SV(leaf, REALSXP, in_nrow);
+		const SparseVec sv2 = leaf2SV(leaf, REALSXP, in_nrow, 0);
 		crossprod2_doublemat_doubleSV(mat1, &sv2, in_nrow,
 					      out, out_nrow);
 	}
@@ -590,7 +564,7 @@ static void crossprod2_mat0_SVT_double(SEXP SVT2, int in_nrow,
 		SEXP leaf = VECTOR_ELT(SVT2, j);
 		if (leaf == R_NilValue)
 			continue;
-		const SparseVec sv = leaf2SV(leaf, REALSXP, in_nrow);
+		const SparseVec sv = leaf2SV(leaf, REALSXP, in_nrow, 0);
 		fill_col(out, out_nrow, _dotprod_doubleSV_zero(&sv));
 	}
 	return;
@@ -609,7 +583,7 @@ static void crossprod2_SVT_mat0_double(SEXP SVT1, int in_nrow,
 		SEXP leaf = VECTOR_ELT(SVT1, i);
 		if (leaf == R_NilValue)
 			continue;
-		const SparseVec sv = leaf2SV(leaf, REALSXP, in_nrow);
+		const SparseVec sv = leaf2SV(leaf, REALSXP, in_nrow, 0);
 		fill_row(out, out_nrow, out_ncol, _dotprod_doubleSV_zero(&sv));
 	}
 	return;
@@ -626,7 +600,7 @@ static void crossprod2_mat0_SVT_int(SEXP SVT2, int in_nrow,
 		SEXP leaf = VECTOR_ELT(SVT2, j);
 		if (leaf == R_NilValue)
 			continue;
-		const SparseVec sv = leaf2SV(leaf, INTSXP, in_nrow);
+		const SparseVec sv = leaf2SV(leaf, INTSXP, in_nrow, 0);
 		if (intSV_has_no_NA(&sv))
 			continue;
 		fill_col(out, out_nrow, NA_REAL);
@@ -645,7 +619,7 @@ static void crossprod2_SVT_mat0_int(SEXP SVT1, int in_nrow,
 		SEXP leaf = VECTOR_ELT(SVT1, i);
 		if (leaf == R_NilValue)
 			continue;
-		const SparseVec sv = leaf2SV(leaf, INTSXP, in_nrow);
+		const SparseVec sv = leaf2SV(leaf, INTSXP, in_nrow, 0);
 		if (intSV_has_no_NA(&sv))
 			continue;
 		fill_row(out, out_nrow, out_ncol, NA_REAL);
@@ -665,10 +639,10 @@ static void compute_dotprods2_with_left_double_leaf(SEXP leaf1, SEXP SVT2,
 						   out, out_nrow, out_ncol);
 		return;
 	}
-	const SparseVec sv1 = leaf2SV(leaf1, REALSXP, dense_len);
+	const SparseVec sv1 = leaf2SV(leaf1, REALSXP, dense_len, 0);
 	if (doubleSV_has_no_NaN_or_Inf(&sv1)) {
 		/* Turn 'sv1' into dense vector. */
-		expand_doubleSV(&sv1, densebuf);
+		_expand_doubleSV(&sv1, densebuf, 1);
 		compute_dotprods2_with_finite_Lcol(densebuf, dense_len, SVT2,
 						   out, out_nrow, out_ncol);
 		return;
@@ -689,10 +663,10 @@ static void compute_dotprods2_with_right_double_leaf(SEXP SVT1, SEXP leaf2,
 						   out, out_nrow);
 		return;
 	}
-	const SparseVec sv2 = leaf2SV(leaf2, REALSXP, dense_len);
+	const SparseVec sv2 = leaf2SV(leaf2, REALSXP, dense_len, 0);
 	if (doubleSV_has_no_NaN_or_Inf(&sv2)) {
 		/* Turn 'sv2' into dense vector. */
-		expand_doubleSV(&sv2, densebuf);
+		_expand_doubleSV(&sv2, densebuf, 1);
 		compute_dotprods2_with_finite_Rcol(SVT1, densebuf, dense_len,
 						   out, out_nrow);
 		return;
@@ -713,10 +687,10 @@ static void compute_dotprods2_with_left_int_leaf(SEXP leaf1, SEXP SVT2,
 						     out, out_nrow, out_ncol);
 		return;
 	}
-	const SparseVec sv1 = leaf2SV(leaf1, INTSXP, dense_len);
+	const SparseVec sv1 = leaf2SV(leaf1, INTSXP, dense_len, 0);
 	if (intSV_has_no_NA(&sv1)) {
 		/* Turn 'sv1' into dense vector. */
-		expand_intSV(&sv1, densebuf);
+		_expand_intSV(&sv1, densebuf, 1);
 		compute_dotprods2_with_noNA_int_Lcol(densebuf, dense_len, SVT2,
 						     out, out_nrow, out_ncol);
 		return;
@@ -737,10 +711,10 @@ static void compute_dotprods2_with_right_int_leaf(SEXP SVT1, SEXP leaf2,
 						     out, out_nrow);
 		return;
 	}
-	const SparseVec sv2 = leaf2SV(leaf2, INTSXP, dense_len);
+	const SparseVec sv2 = leaf2SV(leaf2, INTSXP, dense_len, 0);
 	if (intSV_has_no_NA(&sv2)) {
 		/* Turn 'sv2' into dense vector. */
-		expand_intSV(&sv2, densebuf);
+		_expand_intSV(&sv2, densebuf, 1);
 		compute_dotprods2_with_noNA_int_Rcol(SVT1, densebuf, dense_len,
 						     out, out_nrow);
 		return;
@@ -861,10 +835,10 @@ static void compute_sym_dotprods_double(SEXP SVT, int j,
 						out, out_ncol);
 		return;
 	}
-	const SparseVec sv = leaf2SV(leaf, REALSXP, dense_len);
+	const SparseVec sv = leaf2SV(leaf, REALSXP, dense_len, 0);
 	if (doubleSV_has_no_NaN_or_Inf(&sv)) {
 		/* Turn 'sv' into dense vector. */
-		expand_doubleSV(&sv, densebuf);
+		_expand_doubleSV(&sv, densebuf, 1);
 		*out = _dotprod_doubleSV_finite_doubles(&sv, densebuf);
 		compute_sym_dotprods_with_finite_col(SVT, j,
 						densebuf, dense_len,
@@ -887,10 +861,10 @@ static void compute_sym_dotprods_int(SEXP SVT, int j,
 						out, out_ncol);
 		return;
 	}
-	const SparseVec sv = leaf2SV(leaf, INTSXP, dense_len);
+	const SparseVec sv = leaf2SV(leaf, INTSXP, dense_len, 0);
 	if (intSV_has_no_NA(&sv)) {
 		/* Turn 'sv' into dense vector. */
-		expand_intSV(&sv, densebuf);
+		_expand_intSV(&sv, densebuf, 1);
 		*out = _dotprod_intSV_noNA_ints(&sv, densebuf);
 		compute_sym_dotprods_with_noNA_int_col(SVT, j,
 						densebuf, dense_len,
@@ -939,15 +913,12 @@ static void crossprod1_int(SEXP SVT, int in_nrow, double *out, int out_ncol)
  * and C_crossprod1_SVT()
  */
 
-static SEXPTYPE get_and_check_input_Rtype(SEXP type, const char *what)
+static SEXPTYPE get_and_check_input_Rtype(SEXP type, const char *argname)
 {
 	SEXPTYPE Rtype;
 
-	Rtype = _get_Rtype_from_Rstring(type);
-	if (Rtype == 0)
-		error("SparseArray internal error in "
-		      "get_and_check_input_Rtype():\n"
-		      "    invalid '%s' value", what);
+	Rtype = _get_and_check_Rtype_from_Rstring(type,
+				"get_and_check_input_Rtype", argname);
 	if (Rtype != REALSXP && Rtype != INTSXP)
 		error("SparseArray internal error in "
 		      "get_and_check_input_Rtype():\n"
@@ -986,11 +957,8 @@ SEXP C_crossprod2_SVT_mat(SEXP x_dim, SEXP x_type, SEXP x_SVT, SEXP y,
 		      "    'x_Rtype != TYPEOF(y)' not supported yet");
 
 	/* Check 'ans_type'. */
-	ans_Rtype = _get_Rtype_from_Rstring(ans_type);
-	if (ans_Rtype == 0)
-		error("SparseArray internal error in "
-		      "C_crossprod2_SVT_mat():\n"
-		      "    invalid 'ans_type' value");
+	ans_Rtype = _get_and_check_Rtype_from_Rstring(ans_type,
+					"C_crossprod2_SVT_mat", "ans_type");
 	if (ans_Rtype != REALSXP)
 		error("SparseArray internal error in "
 		      "C_crossprod2_SVT_mat():\n"
@@ -1041,11 +1009,8 @@ SEXP C_crossprod2_mat_SVT(SEXP x, SEXP y_dim, SEXP y_type, SEXP y_SVT,
 		error("input objects must have the same type() for now");
 
 	/* Check 'ans_type'. */
-	ans_Rtype = _get_Rtype_from_Rstring(ans_type);
-	if (ans_Rtype == 0)
-		error("SparseArray internal error in "
-		      "C_crossprod2_mat_SVT():\n"
-		      "    invalid 'ans_type' value");
+	ans_Rtype = _get_and_check_Rtype_from_Rstring(ans_type,
+				"C_crossprod2_mat_SVT", "ans_type");
 	if (ans_Rtype != REALSXP)
 		error("SparseArray internal error in "
 		      "C_crossprod2_mat_SVT():\n"
@@ -1096,11 +1061,8 @@ SEXP C_crossprod2_SVT_SVT(SEXP x_dim, SEXP x_type, SEXP x_SVT,
 		      "must have the same type() for now");
 
 	/* Check 'ans_type'. */
-	ans_Rtype = _get_Rtype_from_Rstring(ans_type);
-	if (ans_Rtype == 0)
-		error("SparseArray internal error in "
-		      "C_crossprod2_SVT_SVT():\n"
-		      "    invalid 'ans_type' value");
+	ans_Rtype = _get_and_check_Rtype_from_Rstring(ans_type,
+				"C_crossprod2_SVT_SVT", "ans_type");
 	if (ans_Rtype != REALSXP)
 		error("SparseArray internal error in "
 		      "C_crossprod2_SVT_SVT():\n"
@@ -1156,11 +1118,8 @@ SEXP C_crossprod1_SVT(SEXP x_dim, SEXP x_type, SEXP x_SVT,
 	x_Rtype = get_and_check_input_Rtype(x_type, "x_type");
 
 	/* Check 'ans_type'. */
-	ans_Rtype = _get_Rtype_from_Rstring(ans_type);
-	if (ans_Rtype == 0)
-		error("SparseArray internal error in "
-		      "C_crossprod1_SVT():\n"
-		      "    invalid 'ans_type' value");
+	ans_Rtype = _get_and_check_Rtype_from_Rstring(ans_type,
+				"C_crossprod1_SVT", "ans_type");
 	if (ans_Rtype != REALSXP)
 		error("SparseArray internal error in "
 		      "C_crossprod1_SVT():\n"
